@@ -1,22 +1,24 @@
 """A class to manage creating image content hashes, and calculate hamming distances"""
-
+#
 # Copyright 2013 Anthony Beville
-
+#
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-
+#
 #     http://www.apache.org/licenses/LICENSE-2.0
-
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 import io
 import logging
 from functools import reduce
+from typing import TypeVar
 
 try:
     from PIL import Image
@@ -28,12 +30,12 @@ logger = logging.getLogger(__name__)
 
 
 class ImageHasher:
-    def __init__(self, path=None, data=None, width=8, height=8):
+    def __init__(self, path: str | None = None, data: bytes = b"", width: int = 8, height: int = 8) -> None:
         self.width = width
         self.height = height
 
-        if path is None and data is None:
-            raise IOError
+        if path is None and not data:
+            raise OSError
 
         try:
             if path is not None:
@@ -45,35 +47,33 @@ class ImageHasher:
             # just generate a bogus image
             self.image = Image.new("L", (1, 1))
 
-    def average_hash(self):
+    def average_hash(self) -> int:
         try:
-            image = self.image.resize((self.width, self.height), Image.ANTIALIAS).convert("L")
+            image = self.image.resize((self.width, self.height), Image.Resampling.LANCZOS).convert("L")
         except Exception:
             logger.exception("average_hash error")
-            return int(0)
+            return 0
 
         pixels = list(image.getdata())
         avg = sum(pixels) / len(pixels)
 
-        def compare_value_to_avg(i):
+        def compare_value_to_avg(i: int) -> int:
             return 1 if i > avg else 0
 
         bitlist = list(map(compare_value_to_avg, pixels))
 
         # build up an int value from the bit list, one bit at a time
-        def set_bit(x, idx_val):
+        def set_bit(x: int, idx_val: tuple[int, int]) -> int:
             (idx, val) = idx_val
             return x | (val << idx)
 
         result = reduce(set_bit, enumerate(bitlist), 0)
 
-        # print("{0:016x}".format(result))
         return result
 
-    def average_hash2(self):
+    def average_hash2(self) -> None:
         """
-        # Got this one from somewhere on the net.  Not a clue how the 'convolve2d'
-        # works!
+        # Got this one from somewhere on the net.  Not a clue how the 'convolve2d' works!
 
         from numpy import array
         from scipy.signal import convolve2d
@@ -87,11 +87,10 @@ class ImageHasher:
         result = reduce(lambda x, (y, z): x | (z << y),
                          enumerate(map(lambda i: 0 if i < 0 else 1, filt_data)),
                          0)
-        #print("{0:016x}".format(result))
         return result
         """
 
-    def dct_average_hash(self):
+    def dct_average_hash(self) -> None:
         """
         # Algorithm source: http://syntaxcandy.blogspot.com/2012/08/perceptual-hash.html
 
@@ -165,14 +164,15 @@ class ImageHasher:
         result = reduce(set_bit, enumerate(bitlist), long(0))
 
 
-        #print("{0:016x}".format(result))
         return result
         """
 
     # accepts 2 hashes (longs or hex strings) and returns the hamming distance
 
+    T = TypeVar("T", int, str)
+
     @staticmethod
-    def hamming_distance(h1, h2):
+    def hamming_distance(h1: T, h2: T) -> int:
         if isinstance(h1, int) or isinstance(h2, int):
             n1 = h1
             n2 = h2
