@@ -1,23 +1,26 @@
 """A class to automatically identify a comic archive"""
-
+#
 # Copyright 2012-2014 Anthony Beville
-
+#
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-
+#
 #     http://www.apache.org/licenses/LICENSE-2.0
-
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 import io
 import logging
 import sys
-from typing import List, TypedDict
+from typing import Any, Callable
+
+from typing_extensions import NotRequired, TypedDict
 
 from comicapi import utils
 from comicapi.comicarchive import ComicArchive
@@ -27,6 +30,7 @@ from comictaggerlib.comicvinetalker import ComicVineTalker, ComicVineTalkerExcep
 from comictaggerlib.imagefetcher import ImageFetcher, ImageFetcherException
 from comictaggerlib.imagehasher import ImageHasher
 from comictaggerlib.resulttypes import IssueResult
+from comictaggerlib.settings import ComicTaggerSettings
 
 logger = logging.getLogger(__name__)
 
@@ -39,19 +43,25 @@ except ImportError:
 
 
 class SearchKeys(TypedDict):
-    series: str
-    issue_number: str
-    month: int
-    year: int
-    issue_count: int
+    series: str | None
+    issue_number: str | None
+    month: int | None
+    year: int | None
+    issue_count: int | None
+
+
+class Score(TypedDict):
+    score: NotRequired[int]
+    url: str
+    hash: int
 
 
 class IssueIdentifierNetworkError(Exception):
-    pass
+    ...
 
 
 class IssueIdentifierCancelled(Exception):
-    pass
+    ...
 
 
 class IssueIdentifier:
@@ -62,7 +72,8 @@ class IssueIdentifier:
     result_one_good_match = 4
     result_multiple_good_matches = 5
 
-    def __init__(self, comic_archive: ComicArchive, settings):
+    def __init__(self, comic_archive: ComicArchive, settings: ComicTaggerSettings) -> None:
+        self.settings = settings
         self.comic_archive: ComicArchive = comic_archive
         self.image_hasher = 1
 
@@ -84,69 +95,69 @@ class IssueIdentifier:
 
         # used to eliminate series names that are too long based on our search
         # string
-        self.length_delta_thresh = settings.id_length_delta_thresh
+        self.series_match_thresh = settings.id_series_match_identify_thresh
 
         # used to eliminate unlikely publishers
-        self.publisher_filter = [s.strip().lower() for s in settings.id_publisher_filter.split(",")]
+        self.publisher_filter = [s.strip().casefold() for s in settings.id_publisher_filter.split(",")]
 
         self.additional_metadata = GenericMetadata()
-        self.output_function = IssueIdentifier.default_write_output
-        self.callback = None
-        self.cover_url_callback = None
+        self.output_function: Callable[[str], None] = IssueIdentifier.default_write_output
+        self.callback: Callable[[int, int], None] | None = None
+        self.cover_url_callback: Callable[[bytes], None] | None = None
         self.search_result = self.result_no_matches
         self.cover_page_index = 0
         self.cancel = False
         self.wait_and_retry_on_rate_limit = False
 
-        self.match_list = []
+        self.match_list: list[IssueResult] = []
 
-    def set_score_min_threshold(self, thresh: int):
+    def set_score_min_threshold(self, thresh: int) -> None:
         self.min_score_thresh = thresh
 
-    def set_score_min_distance(self, distance):
+    def set_score_min_distance(self, distance: int) -> None:
         self.min_score_distance = distance
 
-    def set_additional_metadata(self, md):
+    def set_additional_metadata(self, md: GenericMetadata) -> None:
         self.additional_metadata = md
 
-    def set_name_length_delta_threshold(self, delta):
-        self.length_delta_thresh = delta
+    def set_name_series_match_threshold(self, delta: int) -> None:
+        self.series_match_thresh = delta
 
-    def set_publisher_filter(self, flt):
+    def set_publisher_filter(self, flt: list[str]) -> None:
         self.publisher_filter = flt
 
-    def set_hasher_algorithm(self, algo):
+    def set_hasher_algorithm(self, algo: int) -> None:
         self.image_hasher = algo
 
-    def set_output_function(self, func):
+    def set_output_function(self, func: Callable[[str], None]) -> None:
         self.output_function = func
 
-    def calculate_hash(self, image_data):
-        if self.image_hasher == "3":
-            return ImageHasher(data=image_data).dct_average_hash()
-        if self.image_hasher == "2":
-            return ImageHasher(data=image_data).average_hash2()
+    def calculate_hash(self, image_data: bytes) -> int:
+        if self.image_hasher == 3:
+            return -1  # ImageHasher(data=image_data).dct_average_hash()
+        if self.image_hasher == 2:
+            return -1  # ImageHasher(data=image_data).average_hash2()
 
         return ImageHasher(data=image_data).average_hash()
 
-    def get_aspect_ratio(self, image_data):
+    def get_aspect_ratio(self, image_data: bytes) -> float:
         try:
             im = Image.open(io.BytesIO(image_data))
             w, h = im.size
             return float(h) / float(w)
-        except:
+        except Exception:
             return 1.5
 
-    def crop_cover(self, image_data):
+    def crop_cover(self, image_data: bytes) -> bytes:
 
         im = Image.open(io.BytesIO(image_data))
         w, h = im.size
 
         try:
             cropped_im = im.crop((int(w / 2), 0, w, h))
-        except:
+        except Exception:
             logger.exception("cropCover() error")
-            return None
+            return b""
 
         output = io.BytesIO()
         cropped_im.save(output, format="PNG")
@@ -155,93 +166,72 @@ class IssueIdentifier:
 
         return cropped_image_data
 
-    def set_progress_callback(self, cb_func):
+    def set_progress_callback(self, cb_func: Callable[[int, int], None]) -> None:
         self.callback = cb_func
 
-    def set_cover_url_callback(self, cb_func):
+    def set_cover_url_callback(self, cb_func: Callable[[bytes], None]) -> None:
         self.cover_url_callback = cb_func
 
-    def get_search_keys(self):
+    def get_search_keys(self) -> SearchKeys:
 
         ca = self.comic_archive
-        search_keys: SearchKeys = {
-            "series": None,
-            "issue_number": None,
-            "month": None,
-            "year": None,
-            "issue_count": None,
-        }
 
         if ca is None:
             return None
-
+        search_keys: SearchKeys
         if self.only_use_additional_meta_data:
-            search_keys["series"] = self.additional_metadata.series
-            search_keys["issue_number"] = self.additional_metadata.issue
-            search_keys["year"] = self.additional_metadata.year
-            search_keys["month"] = self.additional_metadata.month
-            search_keys["issue_count"] = self.additional_metadata.issue_count
+            search_keys = SearchKeys(
+                series=self.additional_metadata.series,
+                issue_number=self.additional_metadata.issue,
+                year=self.additional_metadata.year,
+                month=self.additional_metadata.month,
+                issue_count=self.additional_metadata.issue_count,
+            )
             return search_keys
 
         # see if the archive has any useful meta data for searching with
-        if ca.has_cix():
-            internal_metadata = ca.read_cix()
-        elif ca.has_cbi():
-            internal_metadata = ca.read_cbi()
-        else:
-            internal_metadata = ca.read_cbi()
+        try:
+            if ca.has_cix():
+                internal_metadata = ca.read_cix()
+            else:
+                internal_metadata = ca.read_cbi()
+        except Exception as e:
+            internal_metadata = GenericMetadata()
+            logger.error("Failed to load metadata for %s: %s", ca.path, e)
 
         # try to get some metadata from filename
-        md_from_filename = ca.metadata_from_filename()
+        md_from_filename = ca.metadata_from_filename(
+            self.settings.complicated_parser,
+            self.settings.remove_c2c,
+            self.settings.remove_fcbd,
+            self.settings.remove_publisher,
+        )
+
+        working_md = md_from_filename.copy()
+
+        working_md.overlay(internal_metadata)
+        working_md.overlay(self.additional_metadata)
 
         # preference order:
         # 1. Additional metadata
         # 1. Internal metadata
         # 1. Filename metadata
-
-        if self.additional_metadata.series is not None:
-            search_keys["series"] = self.additional_metadata.series
-        elif internal_metadata.series is not None:
-            search_keys["series"] = internal_metadata.series
-        else:
-            search_keys["series"] = md_from_filename.series
-
-        if self.additional_metadata.issue is not None:
-            search_keys["issue_number"] = self.additional_metadata.issue
-        elif internal_metadata.issue is not None:
-            search_keys["issue_number"] = internal_metadata.issue
-        else:
-            search_keys["issue_number"] = md_from_filename.issue
-
-        if self.additional_metadata.year is not None:
-            search_keys["year"] = self.additional_metadata.year
-        elif internal_metadata.year is not None:
-            search_keys["year"] = internal_metadata.year
-        else:
-            search_keys["year"] = md_from_filename.year
-
-        if self.additional_metadata.month is not None:
-            search_keys["month"] = self.additional_metadata.month
-        elif internal_metadata.month is not None:
-            search_keys["month"] = internal_metadata.month
-        else:
-            search_keys["month"] = md_from_filename.month
-
-        if self.additional_metadata.issue_count is not None:
-            search_keys["issue_count"] = self.additional_metadata.issue_count
-        elif internal_metadata.issue_count is not None:
-            search_keys["issue_count"] = internal_metadata.issue_count
-        else:
-            search_keys["issue_count"] = md_from_filename.issue_count
+        search_keys = SearchKeys(
+            series=working_md.series,
+            issue_number=working_md.issue,
+            year=working_md.year,
+            month=working_md.month,
+            issue_count=working_md.issue_count,
+        )
 
         return search_keys
 
     @staticmethod
-    def default_write_output(text):
+    def default_write_output(text: str) -> None:
         sys.stdout.write(text)
         sys.stdout.flush()
 
-    def log_msg(self, msg: str, newline=True):
+    def log_msg(self, msg: Any, newline: bool = True) -> None:
         msg = str(msg)
         if newline:
             msg += "\n"
@@ -249,16 +239,16 @@ class IssueIdentifier:
 
     def get_issue_cover_match_score(
         self,
-        comic_vine,
-        issue_id,
-        primary_img_url,
-        primary_thumb_url,
-        page_url,
-        local_cover_hash_list,
-        use_remote_alternates=False,
-        use_log=True,
-    ):
-        # local_cover_hash_list is a list of pre-calculated hashs.
+        comic_vine: ComicVineTalker,
+        issue_id: int,
+        primary_img_url: str,
+        primary_thumb_url: str,
+        page_url: str,
+        local_cover_hash_list: list[int],
+        use_remote_alternates: bool = False,
+        use_log: bool = True,
+    ) -> Score:
+        # local_cover_hash_list is a list of pre-calculated hashes.
         # use_remote_alternates - indicates to use alternate covers from CV
 
         try:
@@ -274,10 +264,7 @@ class IssueIdentifier:
         if self.cover_url_callback is not None:
             self.cover_url_callback(url_image_data)
 
-        remote_cover_list = []
-        item = {"url": primary_img_url, "hash": self.calculate_hash(url_image_data)}
-
-        remote_cover_list.append(item)
+        remote_cover_list = [Score(url=primary_img_url, hash=self.calculate_hash(url_image_data))]
 
         if self.cancel:
             raise IssueIdentifierCancelled
@@ -298,8 +285,7 @@ class IssueIdentifier:
                 if self.cover_url_callback is not None:
                     self.cover_url_callback(alt_url_image_data)
 
-                item = {"url": alt_url, "hash": self.calculate_hash(alt_url_image_data)}
-                remote_cover_list.append(item)
+                remote_cover_list.append(Score(url=alt_url, hash=self.calculate_hash(alt_url_image_data)))
 
                 if self.cancel:
                     raise IssueIdentifierCancelled
@@ -314,8 +300,7 @@ class IssueIdentifier:
         for local_cover_hash in local_cover_hash_list:
             for remote_cover_item in remote_cover_list:
                 score = ImageHasher.hamming_distance(local_cover_hash, remote_cover_item["hash"])
-                score_item = {"score": score, "url": remote_cover_item["url"], "hash": remote_cover_item["hash"]}
-                score_list.append(score_item)
+                score_list.append(Score(score=score, url=remote_cover_item["url"], hash=remote_cover_item["hash"]))
                 if use_log:
                     self.log_msg(score, False)
 
@@ -334,9 +319,9 @@ class IssueIdentifier:
 
         return best_score_item
 
-    def search(self) -> List[IssueResult]:
+    def search(self) -> list[IssueResult]:
         ca = self.comic_archive
-        self.match_list: List[IssueResult] = []
+        self.match_list = []
         self.cancel = False
         self.search_result = self.result_no_matches
 
@@ -345,7 +330,7 @@ class IssueIdentifier:
             return self.match_list
 
         if not ca.seems_to_be_a_comic_archive():
-            self.log_msg("Sorry, but " + ca.path + " is not a comic archive!")
+            self.log_msg(f"Sorry, but {ca.path} is not a comic archive!")
             return self.match_list
 
         cover_image_data = ca.get_page(self.cover_page_index)
@@ -380,7 +365,7 @@ class IssueIdentifier:
         if keys["month"] is not None:
             self.log_msg("\tMonth:  " + str(keys["month"]))
 
-        comic_vine = ComicVineTalker()
+        comic_vine = ComicVineTalker(self.settings.id_series_match_search_thresh)
         comic_vine.wait_for_rate_limit = self.wait_and_retry_on_rate_limit
 
         comic_vine.set_log_func(self.output_function)
@@ -415,19 +400,17 @@ class IssueIdentifier:
                 if int(keys["year"]) < int(item["start_year"]):
                     date_approved = False
 
-            # assume that our search name is close to the actual name, say
-            # within ,e.g. 5 chars
-            # sanitize both the search string and the result so that
-            # we are comparing the same type of data
-            shortened_key = utils.sanitize_title(keys["series"])
-            shortened_item_name = utils.sanitize_title(item["name"])
-            if len(shortened_item_name) < (len(shortened_key) + self.length_delta_thresh):
-                length_approved = True
-
+            aliases = []
+            if item["aliases"]:
+                aliases = item["aliases"].split("\n")
+            for name in [item["name"], *aliases]:
+                if utils.titles_match(keys["series"], name, self.series_match_thresh):
+                    length_approved = True
+                    break
             # remove any series from publishers on the filter
             if item["publisher"] is not None:
                 publisher = item["publisher"]["name"]
-                if publisher is not None and publisher.lower() in self.publisher_filter:
+                if publisher is not None and publisher.casefold() in self.publisher_filter:
                     publisher_approved = False
 
             if length_approved and publisher_approved and date_approved:
@@ -510,7 +493,7 @@ class IssueIdentifier:
                     hash_list,
                     use_remote_alternates=False,
                 )
-            except:
+            except Exception:
                 self.match_list = []
                 return self.match_list
 
@@ -555,9 +538,9 @@ class IssueIdentifier:
         self.log_msg(f"Compared to covers in {len(self.match_list)} issue(s):", newline=False)
         self.log_msg(str(lst))
 
-        def print_match(item):
+        def print_match(item: IssueResult) -> None:
             self.log_msg(
-                "-----> {0} #{1} {2} ({3}/{4}) -- score: {5}".format(
+                "-----> {} #{} {} ({}/{}) -- score: {}".format(
                     item["series"],
                     item["issue_number"],
                     item["issue_title"],
@@ -576,8 +559,8 @@ class IssueIdentifier:
             hash_list = [cover_hash]
             if narrow_cover_hash is not None:
                 hash_list.append(narrow_cover_hash)
-            for i in range(1, min(3, ca.get_number_of_pages())):
-                image_data = ca.get_page(i)
+            for page_index in range(1, min(3, ca.get_number_of_pages())):
+                image_data = ca.get_page(page_index)
                 page_hash = self.calculate_hash(image_data)
                 hash_list.append(page_hash)
 
@@ -598,7 +581,7 @@ class IssueIdentifier:
                         hash_list,
                         use_remote_alternates=True,
                     )
-                except:
+                except Exception:
                     self.match_list = []
                     return self.match_list
                 self.log_msg(f"--->{score_item['score']}")
@@ -636,9 +619,9 @@ class IssueIdentifier:
             self.callback(99, 100)
 
         # now pare down list, remove any item more than specified distant from the top scores
-        for item in reversed(self.match_list):
-            if item["distance"] > best_score + self.min_score_distance:
-                self.match_list.remove(item)
+        for match_item in reversed(self.match_list):
+            if match_item["distance"] > best_score + self.min_score_distance:
+                self.match_list.remove(match_item)
 
         # One more test for the case choosing limited series first issue vs a trade with the same cover:
         # if we have a given issue count > 1 and the volume from CV has count==1, remove it from match list
@@ -671,8 +654,8 @@ class IssueIdentifier:
             self.log_msg("More than one likely candidate.")
             self.search_result = self.result_multiple_good_matches
             self.log_msg("--------------------------------------------------------------------------")
-            for item in self.match_list:
-                print_match(item)
+            for match_item in self.match_list:
+                print_match(match_item)
             self.log_msg("--------------------------------------------------------------------------")
 
         return self.match_list
