@@ -1,18 +1,19 @@
 """Settings class for ComicTagger app"""
-
+#
 # Copyright 2012-2014 Anthony Beville
-
+#
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-
+#
 #     http://www.apache.org/licenses/LICENSE-2.0
-
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 import configparser
 import logging
@@ -21,6 +22,8 @@ import pathlib
 import platform
 import sys
 import uuid
+from collections.abc import Iterator
+from typing import TextIO, no_type_check
 
 from comicapi import utils
 
@@ -28,35 +31,33 @@ logger = logging.getLogger(__name__)
 
 
 class ComicTaggerSettings:
-    folder = ""
+    folder: pathlib.Path | str = ""
 
     @staticmethod
-    def get_settings_folder():
+    def get_settings_folder() -> pathlib.Path:
         if not ComicTaggerSettings.folder:
             if platform.system() == "Windows":
-                ComicTaggerSettings.folder = os.path.join(os.environ["APPDATA"], "ComicTagger")
+                ComicTaggerSettings.folder = pathlib.Path(os.environ["APPDATA"]) / "ComicTagger"
             else:
-                ComicTaggerSettings.folder = os.path.join(os.path.expanduser("~"), ".ComicTagger")
+                ComicTaggerSettings.folder = pathlib.Path(os.path.expanduser("~")) / ".ComicTagger"
         return pathlib.Path(ComicTaggerSettings.folder)
 
     @staticmethod
-    def base_dir():
-        if getattr(sys, "frozen", None):
-            return sys._MEIPASS
+    def base_dir() -> pathlib.Path:
+        if getattr(sys, "frozen", False) and hasattr(sys, "_MEIPASS"):
+            return pathlib.Path(sys._MEIPASS)  # type: ignore[attr-defined]
 
         return pathlib.Path(__file__).parent
 
     @staticmethod
-    def get_graphic(filename):
-        graphic_folder = pathlib.Path(os.path.join(ComicTaggerSettings.base_dir(), "graphics"))
-        return os.path.join(graphic_folder, filename)
+    def get_graphic(filename: str | pathlib.Path) -> str:
+        return str(ComicTaggerSettings.base_dir() / "graphics" / filename)
 
     @staticmethod
-    def get_ui_file(filename):
-        ui_folder = os.path.join(ComicTaggerSettings.base_dir(), "ui")
-        return os.path.join(ui_folder, filename)
+    def get_ui_file(filename: str | pathlib.Path) -> pathlib.Path:
+        return ComicTaggerSettings.base_dir() / "ui" / filename
 
-    def set_default_values(self):
+    def __init__(self, folder: str | pathlib.Path | None) -> None:
         # General Settings
         self.rar_exe_path = ""
         self.allow_cbi_in_rar = True
@@ -78,7 +79,8 @@ class ComicTaggerSettings:
         self.last_filelist_sorted_order = 0
 
         # identifier settings
-        self.id_length_delta_thresh = 5
+        self.id_series_match_search_thresh = 90
+        self.id_series_match_identify_thresh = 91
         self.id_publisher_filter = "Panini Comics, Abril, Planeta DeAgostini, Editorial Televisa, Dino Comics"
 
         # Show/ask dialog flags
@@ -86,22 +88,26 @@ class ComicTaggerSettings:
         self.show_disclaimer = True
         self.dont_notify_about_this_version = ""
         self.ask_about_usage_stats = True
-        self.hide_rename_message = False
 
         # filename parsing settings
-        self.parse_scan_info = True
+        self.complicated_parser = False
+        self.remove_c2c = False
+        self.remove_fcbd = False
+        self.remove_publisher = False
 
         # Comic Vine settings
         self.use_series_start_as_volume = False
         self.clear_form_before_populating_from_cv = False
         self.remove_html_tables = False
         self.cv_api_key = ""
+        self.cv_url = ""
+        self.auto_imprint = False
 
         self.sort_series_by_year = True
         self.exact_series_matches_first = True
         self.always_use_publisher_filter = False
 
-        # CBL Tranform settings
+        # CBL Transform settings
 
         self.assume_lone_credit_is_primary = False
         self.copy_characters_to_tags = False
@@ -114,88 +120,12 @@ class ComicTaggerSettings:
         self.apply_cbl_transform_on_bulk_operation = False
 
         # Rename settings
-        self.rename_template = "%series% #%issue% (%year%)"
+        self.rename_template = "{series} #{issue} ({year})"
         self.rename_issue_number_padding = 3
         self.rename_use_smart_string_cleanup = True
         self.rename_extension_based_on_archive = True
         self.rename_dir = ""
         self.rename_move_dir = False
-        self.rename_new_renamer = False
-        self.rename_strict = False
-
-        # Auto-tag stickies
-        self.save_on_low_confidence = False
-        self.dont_use_year_when_identifying = False
-        self.assume_1_if_no_issue_num = False
-        self.ignore_leading_numbers_in_filename = False
-        self.remove_archive_after_successful_match = False
-        self.wait_and_retry_on_rate_limit = False
-
-    def __init__(self, folder):
-        # General Settings
-        self.rar_exe_path = ""
-        self.allow_cbi_in_rar = True
-        self.check_for_new_version = False
-        self.send_usage_stats = False
-
-        # automatic settings
-        self.install_id = uuid.uuid4().hex
-        self.last_selected_save_data_style = 0
-        self.last_selected_load_data_style = 0
-        self.last_opened_folder = ""
-        self.last_main_window_width = 0
-        self.last_main_window_height = 0
-        self.last_main_window_x = 0
-        self.last_main_window_y = 0
-        self.last_form_side_width = -1
-        self.last_list_side_width = -1
-        self.last_filelist_sorted_column = -1
-        self.last_filelist_sorted_order = 0
-
-        # identifier settings
-        self.id_length_delta_thresh = 5
-        self.id_publisher_filter = "Panini Comics, Abril, Planeta DeAgostini, Editorial Televisa, Dino Comics"
-
-        # Show/ask dialog flags
-        self.ask_about_cbi_in_rar = True
-        self.show_disclaimer = True
-        self.dont_notify_about_this_version = ""
-        self.ask_about_usage_stats = True
-        self.hide_rename_message = False
-
-        # filename parsing settings
-        self.parse_scan_info = True
-
-        # Comic Vine settings
-        self.use_series_start_as_volume = False
-        self.clear_form_before_populating_from_cv = False
-        self.remove_html_tables = False
-        self.cv_api_key = ""
-
-        self.sort_series_by_year = True
-        self.exact_series_matches_first = True
-        self.always_use_publisher_filter = False
-
-        # CBL Tranform settings
-
-        self.assume_lone_credit_is_primary = False
-        self.copy_characters_to_tags = False
-        self.copy_teams_to_tags = False
-        self.copy_locations_to_tags = False
-        self.copy_storyarcs_to_tags = False
-        self.copy_notes_to_comments = False
-        self.copy_weblink_to_comments = False
-        self.apply_cbl_transform_on_cv_import = False
-        self.apply_cbl_transform_on_bulk_operation = False
-
-        # Rename settings
-        self.rename_template = "%series% #%issue% (%year%)"
-        self.rename_issue_number_padding = 3
-        self.rename_use_smart_string_cleanup = True
-        self.rename_extension_based_on_archive = True
-        self.rename_dir = ""
-        self.rename_move_dir = False
-        self.rename_new_renamer = False
         self.rename_strict = False
 
         # Auto-tag stickies
@@ -232,9 +162,12 @@ class ComicTaggerSettings:
                 elif os.path.exists(r"C:\Program Files (x86)\WinRAR\Rar.exe"):
                     self.rar_exe_path = r"C:\Program Files (x86)\WinRAR\Rar.exe"
             else:
+                if os.path.exists("/opt/homebrew/bin"):
+                    utils.add_to_path("/opt/homebrew/bin")
                 # see if it's in the path of unix user
-                if utils.which("rar") is not None:
-                    self.rar_exe_path = utils.which("rar")
+                rarpath = utils.which("rar")
+                if rarpath is not None:
+                    self.rar_exe_path = "rar"
             if self.rar_exe_path != "":
                 self.save()
         if self.rar_exe_path != "":
@@ -243,16 +176,16 @@ class ComicTaggerSettings:
 
     def reset(self):
         os.unlink(self.settings_file)
-        self.__init__()
+        self.__init__(ComicTaggerSettings.folder)
 
-    def load(self):
-        def readline_generator(f):
+    def load(self) -> None:
+        def readline_generator(f: TextIO) -> Iterator[str]:
             line = f.readline()
             while line:
                 yield line
                 line = f.readline()
 
-        with open(self.settings_file, "r", encoding="utf-8") as f:
+        with open(self.settings_file, encoding="utf-8") as f:
             self.config.read_file(readline_generator(f))
 
         self.rar_exe_path = self.config.get("settings", "rar_exe_path")
@@ -286,13 +219,21 @@ class ComicTaggerSettings:
         if self.config.has_option("auto", "last_filelist_sorted_order"):
             self.last_filelist_sorted_order = self.config.getint("auto", "last_filelist_sorted_order")
 
-        if self.config.has_option("identifier", "id_length_delta_thresh"):
-            self.id_length_delta_thresh = self.config.getint("identifier", "id_length_delta_thresh")
+        if self.config.has_option("identifier", "id_series_match_search_thresh"):
+            self.id_series_match_search_thresh = self.config.getint("identifier", "id_series_match_search_thresh")
+        if self.config.has_option("identifier", "id_series_match_identify_thresh"):
+            self.id_series_match_identify_thresh = self.config.getint("identifier", "id_series_match_identify_thresh")
         if self.config.has_option("identifier", "id_publisher_filter"):
             self.id_publisher_filter = self.config.get("identifier", "id_publisher_filter")
 
-        if self.config.has_option("filenameparser", "parse_scan_info"):
-            self.parse_scan_info = self.config.getboolean("filenameparser", "parse_scan_info")
+        if self.config.has_option("filenameparser", "complicated_parser"):
+            self.complicated_parser = self.config.getboolean("filenameparser", "complicated_parser")
+        if self.config.has_option("filenameparser", "remove_c2c"):
+            self.remove_c2c = self.config.getboolean("filenameparser", "remove_c2c")
+        if self.config.has_option("filenameparser", "remove_fcbd"):
+            self.remove_fcbd = self.config.getboolean("filenameparser", "remove_fcbd")
+        if self.config.has_option("filenameparser", "remove_publisher"):
+            self.remove_publisher = self.config.getboolean("filenameparser", "remove_publisher")
 
         if self.config.has_option("dialogflags", "ask_about_cbi_in_rar"):
             self.ask_about_cbi_in_rar = self.config.getboolean("dialogflags", "ask_about_cbi_in_rar")
@@ -302,8 +243,6 @@ class ComicTaggerSettings:
             self.dont_notify_about_this_version = self.config.get("dialogflags", "dont_notify_about_this_version")
         if self.config.has_option("dialogflags", "ask_about_usage_stats"):
             self.ask_about_usage_stats = self.config.getboolean("dialogflags", "ask_about_usage_stats")
-        if self.config.has_option("dialogflags", "hide_rename_message"):
-            self.hide_rename_message = self.config.getboolean("dialogflags", "hide_rename_message")
 
         if self.config.has_option("comicvine", "use_series_start_as_volume"):
             self.use_series_start_as_volume = self.config.getboolean("comicvine", "use_series_start_as_volume")
@@ -324,6 +263,9 @@ class ComicTaggerSettings:
         if self.config.has_option("comicvine", "cv_api_key"):
             self.cv_api_key = self.config.get("comicvine", "cv_api_key")
 
+        if self.config.has_option("comicvine", "cv_url"):
+            self.cv_url = self.config.get("comicvine", "cv_url")
+
         if self.config.has_option("cbl_transform", "assume_lone_credit_is_primary"):
             self.assume_lone_credit_is_primary = self.config.getboolean(
                 "cbl_transform", "assume_lone_credit_is_primary"
@@ -363,8 +305,6 @@ class ComicTaggerSettings:
             self.rename_dir = self.config.get("rename", "rename_dir")
         if self.config.has_option("rename", "rename_move_dir"):
             self.rename_move_dir = self.config.getboolean("rename", "rename_move_dir")
-        if self.config.has_option("rename", "rename_new_renamer"):
-            self.rename_new_renamer = self.config.getboolean("rename", "rename_new_renamer")
         if self.config.has_option("rename", "rename_strict"):
             self.rename_strict = self.config.getboolean("rename", "rename_strict")
 
@@ -384,8 +324,11 @@ class ComicTaggerSettings:
             )
         if self.config.has_option("autotag", "wait_and_retry_on_rate_limit"):
             self.wait_and_retry_on_rate_limit = self.config.getboolean("autotag", "wait_and_retry_on_rate_limit")
+        if self.config.has_option("autotag", "auto_imprint"):
+            self.auto_imprint = self.config.getboolean("autotag", "auto_imprint")
 
-    def save(self):
+    @no_type_check
+    def save(self) -> None:
 
         if not self.config.has_section("settings"):
             self.config.add_section("settings")
@@ -413,7 +356,8 @@ class ComicTaggerSettings:
         if not self.config.has_section("identifier"):
             self.config.add_section("identifier")
 
-        self.config.set("identifier", "id_length_delta_thresh", self.id_length_delta_thresh)
+        self.config.set("identifier", "id_series_match_search_thresh", self.id_series_match_search_thresh)
+        self.config.set("identifier", "id_series_match_identify_thresh", self.id_series_match_identify_thresh)
         self.config.set("identifier", "id_publisher_filter", self.id_publisher_filter)
 
         if not self.config.has_section("dialogflags"):
@@ -423,12 +367,14 @@ class ComicTaggerSettings:
         self.config.set("dialogflags", "show_disclaimer", self.show_disclaimer)
         self.config.set("dialogflags", "dont_notify_about_this_version", self.dont_notify_about_this_version)
         self.config.set("dialogflags", "ask_about_usage_stats", self.ask_about_usage_stats)
-        self.config.set("dialogflags", "hide_rename_message", self.hide_rename_message)
 
         if not self.config.has_section("filenameparser"):
             self.config.add_section("filenameparser")
 
-        self.config.set("filenameparser", "parse_scan_info", self.parse_scan_info)
+        self.config.set("filenameparser", "complicated_parser", self.complicated_parser)
+        self.config.set("filenameparser", "remove_c2c", self.remove_c2c)
+        self.config.set("filenameparser", "remove_fcbd", self.remove_fcbd)
+        self.config.set("filenameparser", "remove_publisher", self.remove_publisher)
 
         if not self.config.has_section("comicvine"):
             self.config.add_section("comicvine")
@@ -442,6 +388,7 @@ class ComicTaggerSettings:
         self.config.set("comicvine", "always_use_publisher_filter", self.always_use_publisher_filter)
 
         self.config.set("comicvine", "cv_api_key", self.cv_api_key)
+        self.config.set("comicvine", "cv_url", self.cv_url)
 
         if not self.config.has_section("cbl_transform"):
             self.config.add_section("cbl_transform")
@@ -455,7 +402,9 @@ class ComicTaggerSettings:
         self.config.set("cbl_transform", "copy_weblink_to_comments", self.copy_weblink_to_comments)
         self.config.set("cbl_transform", "apply_cbl_transform_on_cv_import", self.apply_cbl_transform_on_cv_import)
         self.config.set(
-            "cbl_transform", "apply_cbl_transform_on_bulk_operation", self.apply_cbl_transform_on_bulk_operation
+            "cbl_transform",
+            "apply_cbl_transform_on_bulk_operation",
+            self.apply_cbl_transform_on_bulk_operation,
         )
 
         if not self.config.has_section("rename"):
@@ -467,7 +416,6 @@ class ComicTaggerSettings:
         self.config.set("rename", "rename_extension_based_on_archive", self.rename_extension_based_on_archive)
         self.config.set("rename", "rename_dir", self.rename_dir)
         self.config.set("rename", "rename_move_dir", self.rename_move_dir)
-        self.config.set("rename", "rename_new_renamer", self.rename_new_renamer)
         self.config.set("rename", "rename_strict", self.rename_strict)
 
         if not self.config.has_section("autotag"):
@@ -478,6 +426,7 @@ class ComicTaggerSettings:
         self.config.set("autotag", "ignore_leading_numbers_in_filename", self.ignore_leading_numbers_in_filename)
         self.config.set("autotag", "remove_archive_after_successful_match", self.remove_archive_after_successful_match)
         self.config.set("autotag", "wait_and_retry_on_rate_limit", self.wait_and_retry_on_rate_limit)
+        self.config.set("autotag", "auto_imprint", self.auto_imprint)
 
         with open(self.settings_file, "w", encoding="utf-8") as configfile:
             self.config.write(configfile)
